"""
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by
 *  Synerty Pty Ltd
 *
"""

import logging
from typing import Dict

from sqlalchemy import MetaData

from peek_plugin_base.storage.DbConnection import DbConnection

logger = logging.getLogger(__name__)

dbConn = None


def setupDbConn(dbConnectString: str, metadata: MetaData, alembicDir: str,
                dbEngineArgs: Dict):
    global dbConn
    dbConn = DbConnection(dbConnectString=dbConnectString,
                          metadata=metadata,
                          alembicDir=alembicDir,
                          dbEngineArgs=dbEngineArgs,
                          enableCreateAll=False,
                          enableForeignKeys=False)


from . import PeekPluginInfo
from . import Setting
from . import PeekEnv



import logging

from peek_platform import PeekPlatformConfig
from peek_platform.sw_version.PeekSwVersionPollHandler import peekPlatformVersionFilt
from peek_platform.sw_version.PeekSwVersionTuple import PeekSwVersionTuple
from peek_server.server.sw_version.PluginSwVersionInfoUtil import \
    getLatestPluginVersionInfos
from vortex.Payload import Payload
from vortex.PayloadEndpoint import PayloadEndpoint
from vortex.VortexABC import SendVortexMsgResponseCallable
from vortex.VortexFactory import VortexFactory

logger = logging.getLogger(__name__)


# -------------------------------------
# Software Update Handler for data from agents

# The filter we listen on

class PeekSwVersionDataHandler(object):
    def __init__(self, payloadFilter):
        self._ep = PayloadEndpoint(payloadFilter, self._process)

    def _process(self, sendResponse: SendVortexMsgResponseCallable, **kwargs):
        data = list()

        # First, name the platform version
        data.append(PeekSwVersionTuple(name="peek_platform",
                                       version=PeekPlatformConfig.config.platformVersion))

        for pluginVersionInfo in getLatestPluginVersionInfos():
            data.append(PeekSwVersionTuple(name=pluginVersionInfo.name,
                                           version=pluginVersionInfo.version))

        sendResponse(Payload(filt=peekPlatformVersionFilt, tuples=data).makePayloadEnvelope().toVortexMsg())

    def notifyOfVersion(self, name, version, vortexUuid=None):
        data = [PeekSwVersionTuple(name=name, version=version)]

        VortexFactory.sendVortexMsg(
            Payload(filt=peekPlatformVersionFilt, tuples=data).makePayloadEnvelope().toVortexMsg(),
            destVortexUuid=vortexUuid)


peekSwVersionDataHandler = PeekSwVersionDataHandler(peekPlatformVersionFilt)



'''
Created on 09/07/2014

@author: synerty
'''

# import logging
# from vortex.DeferUtil import deferToThreadWrapWithLogger
#
# logger = logging.getLogger(__name__)

pluginVersionInfoHandlerKey = {
    'plugin': 'peek_server',
    'key': "peek_server.plugin.version.info"
}

#
# class __PluginVersionInfoHandler(ModelHandler):
#     @deferToThreadWrapWithLogger(logger)
#     def buildModel(self, payloadFilt, **kwargs):
#         return getLatestPluginVersionInfos()
#
#
# __pluginVersionInfoHandler = __PluginVersionInfoHandler(pluginVersionInfoHandlerKey)



# '''
# Created on 09/07/2014
#
# @author: synerty
# '''
#
# # from peek_server.server.live_db.LiveDb import liveDb
# # from peek_server.server.orm.LiveDb import LiveDbKey
# # from peek_server.server.orm.AgentData import AgentImportDispGridInfo
# # from peek_server.server.orm.Display import DispBase
# # from peek_server.server.orm.GridKeyIndex import GridKeyCompilerQueue, DispIndexerQueue
# from txhttputil.util.DeferUtil import deferToThreadWrap
# from vortex.handler.ModelHandler import ModelHandler
#
# import logging
# from vortex.DeferUtil import deferToThreadWrapWithLogger
#
# logger = logging.getLogger(__name__)
#
# executeListDataKey = {'plugin': 'peek_server',
#                       'key': "peakadm.dashboard.list.data"}
#
#
# class DashboardListHandler(ModelHandler):
#
##   @deferToThreadWrapWithLogger(logger)
#     def buildModel(self, payloadFilt, **kwargs):
#         qryItems = []
#
#         from peek_server.storage import dbConn
#         session = dbConn.ormSessionCreator()
#
#         def rowsQuick(Declarative):
#             sql = '''
#                   SELECT 50 * count(*) AS estimate
#                   FROM "%s" TABLESAMPLE SYSTEM (2);
#                   '''
#             sql %= Declarative.__table__.name
#             return session.execute(sql).fetchone()[0]
#
#         def rows(Declarative, quick=False):
#             if quick:
#                 return rowsQuick(Declarative)
#
#             return session.query(Declarative).count()
#
#         # qryItems.append(("Grid Compiler Queue Size", GridKeyCompilerQueue, False))
#         # qryItems.append(("Display Compiler Queue Size", DispIndexerQueue, False))
#         # qryItems.append(("Display Agent Imported Grids", AgentImportDispGridInfo, False))
#         # qryItems.append(("Display Objects (Estimate)", DispBase, True))
#         # qryItems.append(("Live DB Values (Estimate)", LiveDbKey, True))
#
#         session.close()
#
#         data = [dict(desc=desc, value=rows(Declarative, quick))
#                 for desc, Declarative, quick in qryItems]
#
#         # from peek_server.server.queue_processesors.GridKeyQueueCompiler import gridKeyQueueCompiler
#         # data.append(dict(desc="Grid Compiler Status",
#         #                  value=gridKeyQueueCompiler.statusText()))
#         #
#         # from peek_server.server.queue_processesors.DispQueueIndexer import dispQueueCompiler
#         # data.append(dict(desc="Display Compiler Status",
#         #                  value=dispQueueCompiler.statusText()))
#
#         # data.extend([dict(desc=i[0], value=i[1]) for i in liveDb.statusNameValues])
#
#         return data
#
#
# dashboardListHandler = DashboardListHandler(executeListDataKey)



import logging
from typing import Optional, List

from sqlalchemy.sql.functions import func

from peek_server.storage.PeekPluginInfo import PeekPluginInfo

logger = logging.getLogger(__name__)


def getLatestPluginVersionInfos(name: Optional[str] = None) -> List[PeekPluginInfo]:
    """ Get Latest Plugin Version Infos

    This method returns the newest c{PeekPluginInfo} object for each plugin we have
    data for.

    :param name: Optionally, the name of the plugin to get the newest info for.
    :return: An array of c{PeekPluginInfo} objects
    """
    from peek_server.storage import dbConn
    session = dbConn.ormSessionCreator()

    maxGroupedIds = (session
                     .query(func.max(PeekPluginInfo.id).label('maxId'))
                     .group_by(PeekPluginInfo.name)
                     .subquery('maxGroupedIds'))

    qry = (session
           .query(PeekPluginInfo)
           .filter(PeekPluginInfo.id == maxGroupedIds.c.maxId)
           .order_by(PeekPluginInfo.name, PeekPluginInfo.id)
           )

    if name:
        qry = qry.filter(PeekPluginInfo.name == name)

    tuples: List[PeekPluginInfo] = []

    try:
        tuples = qry.all()
        session.expunge_all()

    except Exception as e:
        logger.info("There are no plugin updates")
        # logger.exception(e)

    finally:
        session.close()

    return tuples



"""
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import logging
import os

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql.expression import desc
from twisted.web.server import NOT_DONE_YET
from txhttputil.site.BasicResource import BasicResource
from txhttputil.site.StaticFileResource import StaticFileResource

from peek_platform import PeekPlatformConfig
from peek_server.storage.PeekPluginInfo import PeekPluginInfo

logger = logging.getLogger(__name__)


class PluginSwDownloadResource(BasicResource):
    isLeaf = True
    isGzipped = True

    def render_GET(self, request):
        name = request.args.get(b'name', [None])[0]
        version = request.args.get(b'version', [None])[0]

        if not name and not version:
            msg = "Download requires peek plugin name, Name=%s" % name
            logger.error(msg)
            request.write(msg.encode())
            request.finish()
            return NOT_DONE_YET

        name = name.decode()
        version = version.decode()

        logger.debug("Plugin Download Resource GET, name=%s, version=%s",
                     name, version)

        from peek_server.storage import dbConn
        session = dbConn.ormSession
        qry = session.query(PeekPluginInfo).filter(PeekPluginInfo.name == name)

        try:
            if version:
                pluginInfo = qry.filter(PeekPluginInfo.version == version).one()
            else:
                # Choose the latest
                pluginInfo = qry.order_by(desc(PeekPluginInfo.id)).first()

        except NoResultFound as e:
            logger.warning("There are no packages availible for plugin %s, version %s",
                           name, version)
            request.finish()
            return NOT_DONE_YET

        newSoftwareTar = os.path.join(PeekPlatformConfig.config.pluginSoftwarePath,
                                      pluginInfo.fileName)

        session.close()

        request.responseHeaders.setRawHeaders(b'content-type',
                                              [b'application/octet-stream'])

        resource = StaticFileResource(newSoftwareTar)
        return resource.render_GET(request)



# '''
# Created on 09/07/2014
#
# @author: synerty
# '''
# from peek_platform import PeekPlatformConfig
# from peek_server.backend.navbar.PeekAdmNavbarUserTuple import PeekAdmNavbarUserTuple
# from vortex.handler.ModelHandler import ModelHandler
#
# navbarDataFiltKey = {
#     'plugin': 'peek_server',
#     'key': "nav.adm.user.data"
# }
#
#
# # HANDLER
# class __NavbarHandler(ModelHandler):
#     def buildModel(self,
#                    # payloadFilt=None,
#                    # vortexUuid=None,
#                    # payload=Payload(),
#                    # session=None,
#                    **kwargs):
#
#         navbarTuple = PeekAdmNavbarUserTuple()
#         lic = PeekPlatformConfig.config.capabilities
#
#         navbarTuple.supportExceeded = lic['supportExceeded'] and not lic['demoExceeded']
#         navbarTuple.demoExceeded = lic['demoExceeded']
#         navbarTuple.countsExceeded = lic['countsExceeded']
#         navbarTuple.username = "None"  # userAccess.username
#
#         return [navbarTuple]
#
#
# navbarDataHandler = __NavbarHandler(navbarDataFiltKey)



import logging
import os
import shutil
import sys
import tarfile
from tempfile import NamedTemporaryFile

import jsoncfg
from jsoncfg.value_mappers import require_string
from pytmpdir.Directory import Directory, File
from twisted.internet.defer import inlineCallbacks, returnValue

from peek_platform import PeekPlatformConfig
from peek_platform.sw_install.PluginSwInstallManagerABC import PLUGIN_PACKAGE_JSON, \
    PluginSwInstallManagerABC
from peek_platform.util.PtyUtil import spawnSubprocess, logSpawnException, spawnPty
from peek_server.server.sw_version.PeekSwVersionDataHandler import \
    peekSwVersionDataHandler
from peek_server.storage.PeekPluginInfo import PeekPluginInfo

__author__ = 'synerty'

logger = logging.getLogger(__name__)


class PluginSwUploadManager(object):
    def __init__(self):
        pass

    @inlineCallbacks
    def processUpdate(self, namedTempFile):

        if not tarfile.is_tarfile(namedTempFile.name):
            raise Exception("Uploaded archive is not a tar file")

        # We need the file to end in .tar.gz
        # PIP doesn't like it otherwise
        namedTempTarGzFile = NamedTemporaryFile(
            dir=PeekPlatformConfig.config.tmpPath, suffix=".tar.gz")
        shutil.copy(namedTempFile.name, namedTempTarGzFile.name)
        del namedTempFile

        pluginName, pluginVersion, fullNewTarPath = yield self.updateToTarFile(
            namedTempTarGzFile)

        # Cascade this update to all the other Peek environment components
        yield PeekPlatformConfig.pluginSwInstallManager.installAndReload(
            pluginName, pluginVersion, fullNewTarPath)

        # Cascade this update to all the other Peek environment components
        yield peekSwVersionDataHandler.notifyOfVersion(pluginName, pluginVersion)

        returnValue("%s, %s" % (pluginName, pluginVersion))

    def updateToTarFile(self, namedTempTarGzFile):

        dirName = tarfile.open(namedTempTarGzFile.name).getnames()[0]

        directory = Directory()
        tarfile.open(namedTempTarGzFile.name).extractall(directory.path)
        directory.scan()

        # CHECK
        pgkName, pkgVersion = PluginSwInstallManagerABC.getPackageInfo(directory)

        # CHECK
        pluginPackageFile = self._getFileForFileName(PLUGIN_PACKAGE_JSON, directory)

        # Example
        """
        {
          "title": "Peek App - Noop",
          "packageName": "plugin_noop",
          "company": "Synerty Pty Ltd",
          "website": "www.synerty.com",
          "version": "#PLUGIN_VER#",
          "buildNumber": "#PLUGIN_BUILD#",
          "buildDate": "#BUILD_DATE#"
        }
        """

        peekAppInfo = PeekPluginInfo()
        peekAppInfo.fileName = "%s.tar.gz" % dirName
        peekAppInfo.dirName = dirName

        packageJson = jsoncfg.load_config(pluginPackageFile.realPath)

        peekAppInfo.title = packageJson.plugin.title(require_string)
        peekAppInfo.name = packageJson.plugin.packageName(require_string)
        peekAppInfo.creator = packageJson.plugin.creator(require_string)
        peekAppInfo.website = packageJson.plugin.website(require_string)
        peekAppInfo.version = packageJson.plugin.version(require_string)
        peekAppInfo.buildNumber = packageJson.plugin.buildNumber(require_string)
        peekAppInfo.buildDate = packageJson.plugin.buildDate(require_string)

        del packageJson  # No longer used

        packageName = peekAppInfo.name

        # CHECK
        # Ensure that the python package name starts with "peek_plugin_"
        if not packageName.startswith("peek_plugin_"):
            raise Exception("papp_package.json plugin.packageName must start with"
                            " 'peek_plugin_',"
                            " It's '%s'" % packageName)

        # CHECK
        if packageName.replace("_", "-") != pgkName:
            raise Exception("PyPI package name VS papp_package.json plugin.packageName"
                            " mismatch, python package name underscores are replaced"
                            " with hyphens to match the PyPI package name"
                            " %s(%s) VS %s"
                            % (packageName.replace("_", "-"), packageName, pgkName))

        self._testPackageUpdate(namedTempTarGzFile.name, packageName)

        # CHECK
        # if pluginPackageFile.path != os.path.join(dirName, pgkName):
        #     raise Exception("Expected %s to be at %s, it's at %s"
        #                     % (PLUGIN_PACKAGE_JSON, dirName, pluginPackageFile.path))

        # CHECK
        if not dirName.startswith(pgkName):
            raise Exception("Peek app name '%s' does not match peek root dir name '%s"
                            % (packageName, dirName))

        # CHECK
        if peekAppInfo.version != pkgVersion:
            raise Exception("PyPI package version VS papp_package.json plugin.version"
                            " mismatch. %s VS %s"
                            % (peekAppInfo.version, pkgVersion))

        # Install the TAR file
        fullNewTarPath = os.path.join(PeekPlatformConfig.config.pluginSoftwarePath,
                                      peekAppInfo.fileName)

        shutil.copy(namedTempTarGzFile.name, fullNewTarPath)

        from peek_server.storage import dbConn
        session = dbConn.ormSession
        existing = (session.query(PeekPluginInfo)
                    .filter(PeekPluginInfo.name == peekAppInfo.name,
                            PeekPluginInfo.version == peekAppInfo.version)
                    .all())
        if existing:
            peekAppInfo.id = existing[0].id
            session.merge(peekAppInfo)
        else:
            session.add(peekAppInfo)

        session.commit()
        session.expunge_all()
        session.close()

        return packageName, pkgVersion, fullNewTarPath

    def _getFileForFileName(self, fileName: str, directory: Directory) -> File:
        """ Get File For FileName

        Get the file from the directory that matches the fileName, this directory
         contains the extracted package.

        :param fileName: The filename to look for.
        :param directory: The directory object where the package is extracted to.
        :return: The File object representing the PKG-INFO

        """
        files = [f for f in directory.files if f.name == fileName]

        if len(files) != 1:
            raise Exception("Uploaded package does not contain exatly 1 %s file, found %s"
                            % (fileName, len(files)))

        return files[0]

    def _testPackageUpdate(self, fileName: str, packageName: str) -> None:
        """ Test Package Update

        :param fileName: The full path to the package file to test install
        :param packageName: The name of the python package to try loading.
            EG "peek_plugin_noop"

        Since we ARE running on the server, we will test install these packages here
         first, this is done by creating a virtualenv.

        Currently we just use pip to try and install the package off line, if it's happy
         we're happy.

        """

        # Create the test virtualenv
        virtualEnvDir = Directory()

        virtExec = os.path.join(os.path.dirname(sys.executable), "virtualenv")
        virtArgsList = [virtExec,
                    # Give the virtual environment access to the global
                    '--system-site-packages',
                    virtualEnvDir.path]
        virtArgs = ' '.join(virtArgsList)

        try:
            spawnSubprocess(virtArgs)
            logger.debug("Vritual env created.")

        except Exception as e:
            logSpawnException(e)
            raise Exception("Failed to create virtualenv for platform test")

        pipExec = os.path.join(virtualEnvDir.path, 'bin', 'pip')

        # Install all the packages from the directory
        pipArgs = [pipExec] + PluginSwInstallManagerABC.makePipArgs(fileName)
        pipArgs = ' '.join(pipArgs)

        try:
            spawnPty(pipArgs)

        except Exception as e:
            logSpawnException(e)

            # Update the detail of the exception and raise it
            raise Exception("Test install of updated plugin package failed.")

        # CHECK
        # Now try to load the package name
        #### This didn't load from the virtualenv...
        # modSpec = find_spec(packageName)
        # if not modSpec:
        #     raise Exception("Testing python import of %s failed, package not found"
        #                     % packageName)

        # Continue normally if it all succeeded
        logger.debug("Peek Plugin package update successfully tested.")



#!/usr/bin/env python
""" 
 * synnova.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This sw_upload is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this sw_upload are reserved by
 *  Synerty Pty Ltd
 *
"""

import logging
import os

from pytmpdir.Directory import DirSettings

from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_server import importPackages
from peek_server.storage import setupDbConn
from peek_server.storage.DeclarativeBase import metadata
from txhttputil.site.FileUploadRequest import FileUploadRequest
from peek_platform.util.LogUtil import setupPeekLogger
from vortex.DeferUtil import vortexLogFailure

setupPeekLogger(peekServerName)

from twisted.internet import reactor

logger = logging.getLogger(__name__)


def setupPlatform():
    from peek_platform import PeekPlatformConfig
    PeekPlatformConfig.componentName = peekServerName

    # Tell the platform classes about our instance of the PeekLoaderBase
    from peek_server.plugin.ServerPluginLoader import ServerPluginLoader
    PeekPlatformConfig.pluginLoader = ServerPluginLoader()

    # The config depends on the componentName, order is important
    from peek_server.PeekServerConfig import PeekServerConfig
    PeekPlatformConfig.config = PeekServerConfig()

    # Set default logging level
    logging.root.setLevel(PeekPlatformConfig.config.loggingLevel)

    # Set paths for the Directory object
    DirSettings.defaultDirChmod = PeekPlatformConfig.config.DEFAULT_DIR_CHMOD
    DirSettings.tmpDirPath = PeekPlatformConfig.config.tmpPath
    FileUploadRequest.tmpFilePath = PeekPlatformConfig.config.tmpPath


def main():
    setupPlatform()
    from peek_platform import PeekPlatformConfig
    import peek_server

    # Configure sqlalchemy
    setupDbConn(
        metadata=metadata,
        dbEngineArgs=PeekPlatformConfig.config.dbEngineArgs,
        dbConnectString=PeekPlatformConfig.config.dbConnectString,
        alembicDir=os.path.join(os.path.dirname(peek_server.__file__), "alembic")
    )

    # Force model migration
    from peek_server.storage import dbConn
    dbConn.migrate()

    # Import remaining components
    importPackages()

    reactor.addSystemEventTrigger('before', 'shutdown',
                                  PeekPlatformConfig.pluginLoader.unloadOptionalPlugins)
    reactor.addSystemEventTrigger('before', 'shutdown',
                                  PeekPlatformConfig.pluginLoader.unloadCorePlugins)

    # Load all plugins
    d = PeekPlatformConfig.pluginLoader.loadCorePlugins()
    d.addCallback(lambda _: PeekPlatformConfig.pluginLoader.loadOptionalPlugins())
    d.addErrback(vortexLogFailure, logger, consumeError=True)

    reactor.run()


if __name__ == '__main__':
    main()



from txhttputil.util.ModuleUtil import filterModules

for mod in filterModules(__name__, __file__):
    __import__(mod, locals(), globals())

from . import sw_download
from . import sw_install
from . import sw_upload
from . import sw_version



""" 
 * subcontent.__init__.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""

import os.path as p

from txhttputil.util.ModuleUtil import filterModules

__modPath = p.dirname(__file__)

for mod in filterModules(__name__, __file__):
    __import__(mod, locals(), globals())




# from peek_platform import PeekPlatformConfig
# from peek_server.backend.navbar.PeekAdmNavbarHandler import navbarDataHandler
# from vortex.DataWrapTuple import DataWrapTuple
# from vortex.Payload import Payload
# from vortex.PayloadEndpoint import PayloadEndpoint
# from vortex.VortexABC import SendVortexMsgResponseCallable
#
# capabilitiesDataFiltKey = {
#     "plugin": "peek_server",
#     "key": "admin.capabilities.data"
# }

#
# # HANDLER
# class __CrudHandler():
#     def __init__(self, payloadFilter):
#         self._payloadFilter = payloadFilter
#         self._ep = PayloadEndpoint(self._payloadFilter, self.process)
#
#     def process(self, payload,
#                 vortexUuid,
#                 httpSession,
#                 sendResponse: SendVortexMsgResponseCallable,
#                               **kwargs):
#         # from peek_server.server.auth import AuthValue
#         from peek_server.storage.Setting import internalSetting, CAPABILITIES_KEY
#
#         result = None
#         from peek_server.storage import dbConn
#         session = dbConn.ormSessionCreator
#
#         # Force capabilities reload on page load
#         PeekPlatformConfig.config._capabilities = None
#
#         # This is an update
#         if payload.tuples:
#             dataWrapTuple = payload.tuples[0]
#             self._capabilities = None
#
#             # try:
#             #     AuthValue.loadCapabilities(self, dataWrapTuple.data)
#             # except Exception as e:
#             #     pass
#             if self._capabilities is None:
#                 result = {"success": False,
#                           "message": "The license entered is not valid for this server"}
#                 sendResponse(Payload(filt=self._payloadFilter,
#                                           result=result).toVortexMsg())
#                 return
#
#             PeekPlatformConfig.config._capabilities = self._capabilities
#
#             internalSetting()[CAPABILITIES_KEY] = dataWrapTuple.data
#             session.commit()
#             result = {"success": True,
#                       "message": "The license was successfully loaded"}
#         dataWrapTuple = DataWrapTuple()
#         dataWrapTuple.data = PeekPlatformConfig.config.capabilities
#
#         sendResponse(Payload(filt=self._payloadFilter,
#                                   tuples=[dataWrapTuple],
#                                   result=result).toVortexMsg())
#
#         navbarDataHandler.sendModelUpdate(vortexUuid=vortexUuid)
#
#
# __ormCrudHandler = __CrudHandler(capabilitiesDataFiltKey)



import os

from txhttputil.site.FileUnderlayResource import FileUnderlayResource

docSiteRoot = FileUnderlayResource()


def setupDocSite():
    # Setup properties for serving the site
    docSiteRoot.enableSinglePageApplication()

    import peek_doc_admin
    docProjectDir = os.path.dirname(peek_doc_admin.__file__)
    distDir = os.path.join(docProjectDir, 'doc_dist')

    buildDirParent = os.path.dirname(distDir)
    if not os.path.isdir(buildDirParent):
        raise NotADirectoryError(buildDirParent)

    # Make the dist dir, otherwise addFileSystemRoot throws an exception.
    # It rebuilds at a later date
    os.makedirs(distDir, exist_ok=True)

    docSiteRoot.addFileSystemRoot(distDir)



from sqlalchemy import engine_from_config, pool

from alembic import context
from sqlalchemy.dialects.mssql.base import MSDialect
from sqlalchemy.dialects.postgresql.base import PGDialect
from peek_platform.util.LogUtil import setupPeekLogger
from peek_plugin_base.PeekVortexUtil import peekServerName

global config
config = context.config

setupPeekLogger(peekServerName)

from peek_server.storage.DeclarativeBase import metadata
target_metadata = metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.

def exclude_tables_from_config(config_):
    tables_ = config_.get("tables", None)
    if tables_ is not None:
        tables = tables_.split(",")
    return tables

exclude_tables = exclude_tables_from_config(config.get_section('alembic:exclude'))


def include_object(object, name, type_, reflected, compare_to):
    # If it's not in this schema, don't include it
    if hasattr(object, 'schema') and object.schema != target_metadata.schema:
        return False

    if type_ == "table" and name in 'spatial_ref_sys':
        return False
    else:
        return True

def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix='sqlalchemy.',
        poolclass=pool.NullPool)

    with connectable.connect() as connection:
        # Ensure the schema exists
        if isinstance(connection.dialect, MSDialect):
            connection.execute("IF(SCHEMA_ID('%s')IS NULL) BEGIN EXEC('CREATE "
                               "SCHEMA [%s]')END" % (target_metadata.schema,
                                                     target_metadata.schema))

        elif isinstance(connection.dialect, PGDialect):
            connection.execute('CREATE SCHEMA IF NOT EXISTS "%s" ' %
                               target_metadata.schema)

        else:
            raise Exception('unknown dialect %s' % connection.dialect)

        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            include_object=include_object,
            include_schemas=True,
            version_table_schema=target_metadata.schema
        )

        with context.begin_transaction():
            context.run_migrations()

if context.is_offline_mode():
    raise NotImplementedError("Peek doesn't allow offline migrations")
else:
    run_migrations_online()



""" 
 * orm.ExtensionData.py
 *
 *  Copyright Synerty Pty Ltd 2011
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import logging

from sqlalchemy import Column
from sqlalchemy import Integer, String
from sqlalchemy.sql.schema import Index

from peek_server.storage.DeclarativeBase import DeclarativeBase
from vortex.Tuple import Tuple, addTupleType

logger = logging.getLogger(__name__)


@addTupleType
class PeekPluginInfo(Tuple, DeclarativeBase):
    """ PeekPluginInfo

    This table stores information on the version of Peek apps that are stored in Peek.

    """
    __tupleType__ = 'peek_server.plugin.info'
    __tablename__ = 'PeekPluginInfo'

    id = Column(Integer, primary_key=True)
    title = Column(String, nullable=False)
    name = Column(String(50), nullable=False)
    version = Column(String(40), nullable=False)
    creator = Column(String, nullable=True)
    website = Column(String, nullable=True)
    buildNumber = Column(String, nullable=True)
    buildDate = Column(String, nullable=True)

    # File location columns
    fileName = Column(String, nullable=False)
    dirName = Column(String, nullable=False)

    __table_args__ = (
        Index("idx_PeekPluginInfo_NameVersion",
              name, version, unique=True),
    )



import logging
from typing import List

import os

from peek_platform.build_doc.DocBuilder import DocBuilder
from peek_platform.build_frontend.WebBuilder import WebBuilder

logger = logging.getLogger(__name__)


class ServerFrontendLoadersMixin:

    def _buildAdminSite(self, loadedPlugins):
        from peek_platform import PeekPlatformConfig

        try:
            import peek_admin
            frontendProjectDir = os.path.dirname(peek_admin.__file__)
        except:
            logger.warning("Skipping builds of peek-admin"
                           ", the package can not be imported")
            return

        webBuilder = WebBuilder(frontendProjectDir,
                                "peek-admin",
                                PeekPlatformConfig.config,
                                loadedPlugins)
        yield webBuilder.build()

    def _buildDocs(self, loadedPlugins):
        # --------------------
        # Prepare the Admin Docs
        from peek_platform import PeekPlatformConfig

        try:
            import peek_doc_admin
            docProjectDir = os.path.dirname(peek_doc_admin.__file__)

        except:
            logger.warning("Skipping builds of peek_doc_admin"
                           ", the package can not be imported")
            return

        docBuilder = DocBuilder(docProjectDir,
                                "peek-doc-admin",
                                PeekPlatformConfig.config,
                                loadedPlugins)
        yield docBuilder.build()

    def _unloadPluginFromAdminSite(self, pluginName):
        # JJC - This doesn't seem right, maybe if it was removing it from the platform
        # JJC - I think this is old code

        # Remove the Plugin resource tree
        from peek_server.backend.AdminSiteResource import adminSiteRoot

        try:
            adminSiteRoot.deleteChild(pluginName.encode())
        except KeyError:
            pass
